-- On the Fence scripting by Mr Nobody
-- v1.2 16 May 22 for Syria

-- Initial implementation of enemy aircraft spawns
-- Slight reworks of the dummy proofing system for ME flags

-- Infantry group templates to spawn
infantryGroups = {
	"Inf Group 1",
	"Inf Group 2",
	"Inf Group 3",
	"Inf Group 4",
	"Inf Group 5",
	"Inf Group 6",
	"Inf Group 7",
}

-- Vehicle group templates to spawn
vehicleGroups = {
	"Veh Group 1",
	"Veh Group 2",
	"Veh Group 3",
	"Veh Group 4",
	"Veh Group 5",
	"Veh Group 6",
	"Veh Group 7",
	"Veh Group 8",
	"Veh Group 9",
	"Veh Group 10",
	"Veh Group 11",
	"Veh Group 12",
	"Veh Group 13",
}

-- Aircraft group templates to spawn
airGroups = {
	"CAP Group 1",
	"CAP Group 2",
	"ATK Group 1",
	"ATK Group 2",
	"ATK Group 3",
}

-- List of trigger zones the enemy can attack
combatZones = {
	"ALPHA",
	"BRAVO",
	"CHARLIE",
	"DELTA",
	"ECHO",
	"FOXTROT",
	"GOLF",
	"HOTEL",
	"INDIA",
	"JULIET",
	"KILO",
	"LIMA",
	"MIKE",
	"NOVEMBER",
}

-- Create an attack if the maximum amount isn't exceeded
function createZoneAttack()

if not activeZones then activeZones = 0 end
if not totalAttacks then totalAttacks = 0 end

-- Pick a zone to attack
zoneSelect = math.random(1, #combatZones)
zoneChoice = combatZones[zoneSelect]
	
-- If the zone already has an enemy presence, the roll also gets canceled to avoid stacking a zone
local enemyPresent = mist.getUnitsInZones(mist.makeUnitTable({'[red][vehicle]'}), {zoneChoice})

	if #enemyPresent == 0 then
		if activeZones < maxActiveZones then
		
		-- Infantry template spawns
		for i = 1 , math.random(minInfantryGroups, maxInfantryGroups) do
			local infGroupSelect = math.random(1, #infantryGroups)
			local infGroupChoice = infantryGroups[infGroupSelect]
			mist.cloneInZone(infGroupChoice, zoneChoice, true, 60)
		end

		-- Vehicle template spawns
		for i = 1 , math.random(minVehicleGroups, maxVehicleGroups) do
			local vehGroupSelect = math.random(1, #vehicleGroups)
			local vehGroupChoice = vehicleGroups[vehGroupSelect]
			mist.cloneInZone(vehGroupChoice, zoneChoice, true, 150)
		end
		
		-- Increment the amount of active combat zones, and the total attacks the enemy has made
		activeZones = activeZones + 1
		totalAttacks = totalAttacks + 1
		
		trigger.action.outText("Enemy forces are attacking " .. zoneChoice .. "!", 15)
		trigger.action.outSound("attack.ogg")
		
		trigger.action.setMarkupColor(zoneSelect, redOutline)
		trigger.action.setMarkupColorFill(zoneSelect, redFill)
		trigger.action.setMarkupText((zoneSelect + #combatZones), "Zone: " .. zoneChoice .. "\nStatus: UNDER ATTACK")
		
		trigger.action.effectSmokeBig(mist.utils.makeVec3GL(trigger.misc.getZone(zoneChoice).point) , 7 , 0.1 , zoneChoice)
		timer.scheduleFunction(checkZoneCompletion , zoneSelect , timer.getTime() +  5)
		env.info("OTF Debug: New zone attack created.")
		else
		env.info("OTF Debug: Active zones at max, skipping spawn process.")
		end
		
		if totalAttacks < maxAttackAmount then
		-- Schedule the next attack on the usual time
			timer.scheduleFunction(createZoneAttack , nil , timer.getTime() +  math.random(avgAttackTime * 0.8, avgAttackTime * 1.2))
		else
			timer.scheduleFunction(finalZoneCheck , nil , timer.getTime() +  30)
		end
		
	else
		-- Schedule the next attack quickly since the roll was a dud
		if totalAttacks < maxAttackAmount then
			timer.scheduleFunction(createZoneAttack , nil , timer.getTime() +  5)
		else
			timer.scheduleFunction(finalZoneCheck , nil , timer.getTime() +  30)
		end
	end
	
end

-- Check to see if Blue outnumbers Red in the air, and if so, spawn one of the aircraft encounters
function createAirEvent()

-- Add a little bit of randomness to the next spawn
local nextAttackTime = math.random(enemyAircraftFrequency * 0.8, enemyAircraftFrequency * 1.2)
local alliedFixedWing = mist.getUnitsInZones(mist.makeUnitTable({"[blue][plane]"}), {"AO"}, "cylinder")
local enemyFixedWing = mist.getUnitsInZones(mist.makeUnitTable({"[red][plane]"}), {"AO"}, "cylinder")
local alliedAmt = #alliedFixedWing
local enemyAmt = #enemyFixedWing

-- Allow an enemy fixed wing event to occur if Blue has more aircraft in the target area
	if alliedAmt > enemyAmt then
		local airGroupSelect = math.random(1, #airGroups)
		local airGroupChoice = airGroups[airGroupSelect]
		mist.cloneGroup(airGroupChoice, true)
		trigger.action.outText("Enemy air activity has been detected near your area of operations.  Be on your guard.\n\n[Tune UHF 264 MHz AM for AWACS 'Darkstar']", 15)
		trigger.action.outSound("airevent.ogg")
		env.info("OTF Debug: Allied fighters have a numbers advantage, air event spawned.")
	else
		env.info("OTF Debug: Allied fighters are outnumbered in the AO, skipping the air event spawn this time.")
	end
timer.scheduleFunction(createAirEvent , nil , timer.getTime() + nextAttackTime)

end

-- Check the active zones table ocasionally to see if they've been cleared of REDFOR
function checkZoneCompletion(zoneId)

local zoneName = combatZones[zoneId]
local attackerUnits = mist.getUnitsInZones(mist.makeUnitTable({'[red][vehicle]'}), {zoneName .. " Large"}) -- Added a slightly larger zone than the spawn one in v1.1 to account for unit dispersion
local attackerUnitsQty = #attackerUnits

	if attackerUnitsQty == 0 then
		trigger.action.outText("We've successfully cleared the enemy from " .. zoneName .. ".  Well done.", 15)
		trigger.action.outSound("clear.ogg")
		trigger.action.setMarkupColor(zoneId, blueOutline)
		trigger.action.setMarkupColorFill(zoneId, blueFill)
		trigger.action.setMarkupText((zoneId + #combatZones), "Zone: " .. zoneName .. "\nStatus: CLEAR")
		trigger.action.effectSmokeStop(zoneName)
		activeZones = activeZones - 1
		env.info("OTF Debug: Zone " .. zoneName .. " reported as clear.")
	elseif attackerUnitsQty > 0 then
		trigger.action.setMarkupText((zoneId + #combatZones), "Zone: " .. zoneName .. "\nStatus: UNDER ATTACK\nEnemy Strength: " .. attackerUnitsQty .. " hostiles")
		timer.scheduleFunction(checkZoneCompletion , zoneId , timer.getTime() +  20)
		env.info("OTF Debug: Zone " .. zoneName .. " reported as under attack, enemy count updated.")
	end
end

-- Check for activeZones to equal 0 periodically once the final wave has spawned
function finalZoneCheck()

-- Only send this message the first time the check occurs
if not finalMessage then
trigger.action.outText("Alright, this looks like the final enemy push.  Finish off all their remaining forces and the day is ours!", 30)
trigger.action.outSound("final.ogg")	
finalMessage = true
end

if activeZones == 0 then
	trigger.action.outText("Enemy forces have been completely wiped out.  Well done, everyone.  Mission is complete, you are cleared to return to base.\n\n === MISSION COMPLETE ===", 60)
	trigger.action.outSound("complete.ogg")
	-- Remove the markers
	for i = 1, #combatZones do
		local markerId = i
		local textId = i + #combatZones
		trigger.action.removeMark(markerId)
		trigger.action.removeMark(textId)		
	end
	trigger.action.setUserFlag("Mission Over", 1)
	env.info("OTF Debug: Mission complete!")
	else
	timer.scheduleFunction(finalZoneCheck , nil , timer.getTime() +  30)
	env.info("OTF Debug: Mission not yet finished, checking again in 30 seconds.")
end

end

-- Automatically mark all the zones
function initMarkZones()

for i = 1, #combatZones do
local markerId = i
local zoneName = combatZones[markerId]
local textId = markerId + #combatZones
local zonePos = trigger.misc.getZone(zoneName)
trigger.action.circleToAll(-1 , markerId , zonePos.point , zonePos.radius , blueOutline , blueFill , 2 , true , "Outpost marked.")
trigger.action.textToAll(-1 , textId , zonePos.point , blackText , clearBackground , 14 , true , "Zone: " .. zoneName .. "\nStatus: CLEAR")
end
env.info("OTF Debug: Combat zones marked on map.")
end

-- Define some variables and sanity check the user options a few seconds after mission start
function otfSetup()

-- Grab the ME flags for user options since they can be named in DCS 2.7.11.  If they've been deleted from the ME by accident, set a default value
if not trigger.misc.getUserFlag("Maximum Active Zones") then maxActiveZones = 3 else maxActiveZones = trigger.misc.getUserFlag("Maximum Active Zones") end
if not trigger.misc.getUserFlag("Time Between Attacks") then avgAttackTime = 600 else avgAttackTime = (trigger.misc.getUserFlag("Time Between Attacks") * 60) end
if not trigger.misc.getUserFlag("Zones Cleared To Win") then maxAttackAmount = 6 else maxAttackAmount = trigger.misc.getUserFlag("Zones Cleared To Win") end
if not trigger.misc.getUserFlag("Minimum Infantry Presence") then minInfantryGroups = 1 else minInfantryGroups = trigger.misc.getUserFlag("Minimum Infantry Presence") end
if not trigger.misc.getUserFlag("Maximum Infantry Presence") then minInfantryGroups = 3 else maxInfantryGroups = trigger.misc.getUserFlag("Maximum Infantry Presence") end
if not trigger.misc.getUserFlag("Minimum Vehicle Presence") then minVehicleGroups = 1 else minVehicleGroups = trigger.misc.getUserFlag("Minimum Vehicle Presence") end
if not trigger.misc.getUserFlag("Maximum Vehicle Presence") then maxVehicleGroups = 2 else maxVehicleGroups = trigger.misc.getUserFlag("Maximum Vehicle Presence") end
if not trigger.misc.getUserFlag("Enemy Aircraft Presence") then enemyAircraft = 0 else enemyAircraft = trigger.misc.getUserFlag("Enemy Aircraft Presence") end
if not trigger.misc.getUserFlag("Enemy Aircraft Frequency") then enemyAircraftFrequency = 600 else enemyAircraftFrequency = (trigger.misc.getUserFlag("Enemy Aircraft Frequency") * 60) end


-- Colors for the map markers
redOutline = {0.95,0.1,0.15,0.7}
redFill = {0.95,0.1,0.15,0.25}
blueOutline = {0,0.7,0.9,0.7}
blueFill = {0,0.7,0.9,0.25}
blackText = {0,0,0,0.7}
clearBackground = {0,0,0,0}

-- stop the user from putting in some really inane or silly values for settings
if maxActiveZones <= 0 then maxActiveZones = 2 end
if avgAttackTime < 60 then avgAttackTime = 600 end
if minInfantryGroups < 0 or minInfantryGroups > 10 then minInfantryGroups = 2 end
if maxInfantryGroups < minInfantryGroups or maxInfantryGroups > 10 then maxInfantryGroups = 4 end
if minVehicleGroups < 0 or minVehicleGroups > 10 then minVehicleGroups = 1 end
if maxVehicleGroups < minVehicleGroups or maxVehicleGroups > 10 then maxVehicleGroups = 3 end
if enemyAircraftFrequency < 60 then enemyAircraftFrequency = 300 end

-- Start the attack generation
timer.scheduleFunction(createZoneAttack , nil , timer.getTime() +  math.random(avgAttackTime * 0.8, avgAttackTime * 1.2))

-- Start the air events if enabled
if enemyAircraft ~= 0 then createAirEvent() else end

-- Draw the initial markers
initMarkZones()
trigger.action.outText("The friendly border outposts have been marked on the map.  We'll keep you updated with any reports of insurgent attacks.  Good luck and stay safe out there.", 60, true)
trigger.action.outSound("intro.ogg")
env.info("OTF Debug: Setup completed!")
end

-- It's showtime!
timer.scheduleFunction(otfSetup , nil , timer.getTime() +  5)
env.info("OTF Debug: Script loaded successfully!")